@@ -17,6 +17,7 @@
 #include "admin/angel/Core.h"
 #include "admin/angel/InterfaceWaiter.h"
 #include "admin/AuthorizedPasswords.h"
+#include "benc/Dict.h"
 #include "benc/Int.h"
 #include "benc/serialization/standard/BencMessageReader.h"
 #include "benc/serialization/standard/BencMessageWriter.h"
@@ -24,33 +25,34 @@
 #include "crypto/random/Random.h"
 #include "crypto/random/nanotime/NanotimeEntropyProvider.h"
 #include "crypto/Sign_admin.h"
+#include "rust/cjdns_sys/RTypes.h"
+#include "subnode/PeeringSeeder_admin.h"
 #include "subnode/SubnodePathfinder.h"
+#include "subnode/SubnodePathfinder_admin.h"
 #include "subnode/SupernodeHunter_admin.h"
 #include "subnode/ReachabilityCollector_admin.h"
+#include "util/platform/Sockaddr.h"
 #ifndef SUBNODE
 #include "dht/Pathfinder.h"
 #endif
-#include "exception/Er.h"
+#include "exception/Err.h"
 #include "interface/Iface.h"
 #include "util/events/UDPAddrIface.h"
 #include "interface/tuntap/TUNInterface.h"
-#include "interface/tuntap/SocketInterface.h"
 #include "interface/tuntap/SocketWrapper.h"
-#include "interface/tuntap/AndroidWrapper.h"
 #include "interface/UDPInterface_admin.h"
 #ifdef HAS_ETH_INTERFACE
 #include "interface/ETHInterface_admin.h"
 #endif
 #include "net/InterfaceController_admin.h"
-#include "interface/addressable/PacketHeaderToUDPAddrIface.h"
 #include "interface/addressable/AddrIfaceMuxer.h"
 #include "interface/tuntap/TUNMessageType.h"
 #include "interface/ASynchronizer.h"
 #include "memory/Allocator.h"
-#include "memory/MallocAllocator.h"
 #include "memory/Allocator_admin.h"
 #include "net/SwitchPinger_admin.h"
 #include "net/UpperDistributor_admin.h"
+#include "rust/cjdns_sys/Rffi.h"
 
 #define NumberCompress_OLD_CODE
 #include "switch/NumberCompress.h"
@@ -58,9 +60,9 @@
 #include "tunnel/IpTunnel_admin.h"
 #include "tunnel/RouteGen_admin.h"
 #include "util/events/EventBase.h"
-#include "util/events/Pipe.h"
-#include "util/events/PipeServer.h"
+#include "util/events/Socket.h"
 #include "util/events/Timeout.h"
+#include "util/events/libuv/Glock.h"
 #include "util/Hex.h"
 #include "util/log/FileWriterLog.h"
 #include "util/log/IndirectLog.h"
@@ -75,8 +77,6 @@
 #include "wire/Headers.h"
 #include "net/NetCore.h"
 
-#include <crypto_scalarmult_curve25519.h>
-
 #include <stdlib.h>
 #include <unistd.h>
 
@@ -101,13 +101,21 @@
 /** The default MTU, assuming the external MTU is 1492 (common for PPPoE DSL) */
 #define DEFAULT_MTU ( 1492 - WORST_CASE_OVERHEAD )
 
-static void adminPing(Dict* input, void* vadmin, String* txid, struct Allocator* requestAlloc)
+static void adminPing(
+    Gcc_UNUSED Dict* input,
+    void* vadmin,
+    String* txid,
+    Gcc_UNUSED struct Allocator* requestAlloc)
 {
     Dict d = Dict_CONST(String_CONST("q"), String_OBJ(String_CONST("pong")), NULL);
     Admin_sendMessage(&d, txid, (struct Admin*) vadmin);
 }
 
-static void adminPid(Dict* input, void* vadmin, String* txid, struct Allocator* requestAlloc)
+static void adminPid(
+    Gcc_UNUSED Dict* input,
+    void* vadmin,
+    String* txid,
+    Gcc_UNUSED struct Allocator* requestAlloc)
 {
     int pid = getpid();
     Dict d = Dict_CONST(String_CONST("pid"), Int_OBJ(pid), NULL);
@@ -119,13 +127,14 @@ struct Context
     struct Allocator* alloc;
     struct Admin* admin;
     struct Log* logger;
-    struct EventBase* base;
+    EventBase_t* base;
     struct NetCore* nc;
     struct IpTunnel* ipTunnel;
     struct EncodingScheme* encodingScheme;
     struct GlobalConfig* globalConf;
 
-    struct Iface* tunDevice;
+    Iface_t* tunIface;
+    TUNInterface_t* tun;
     struct Allocator* tunAlloc;
 
     Identity
@@ -137,7 +146,11 @@ static void shutdown(void* vcontext)
     Allocator_free(context->alloc);
 }
 
-static void adminExit(Dict* input, void* vcontext, String* txid, struct Allocator* requestAlloc)
+static void adminExit(
+    Gcc_UNUSED Dict* input,
+    void* vcontext,
+    String* txid,
+    Gcc_UNUSED struct Allocator* requestAlloc)
 {
     struct Context* context = Identity_check((struct Context*) vcontext);
     Log_info(context->logger, "Got request to exit");
@@ -156,37 +169,36 @@ static void sendResponse(String* error,
     Admin_sendMessage(output, txid, admin);
 }
 
-static Er_DEFUN(void initSocket2(String* socketFullPath,
-                          struct Context* ctx,
-                          uint8_t addressPrefix))
+static Err_DEFUN initSocket2(String* socketFullPath, struct Context* ctx)
 {
     Log_debug(ctx->logger, "Initializing socket: %s;", socketFullPath->bytes);
 
-    if (ctx->tunDevice) {
-        Iface_unplumb(ctx->tunDevice, &ctx->nc->tunAdapt->tunIf);
+    if (ctx->tunIface) {
+        Iface_unplumb(ctx->tunIface, &ctx->nc->tunAdapt->tunIf);
         Allocator_free(ctx->tunAlloc);
-        ctx->tunDevice = NULL;
+        ctx->tunIface = NULL;
+        ctx->tun = NULL;
     }
     ctx->tunAlloc = Allocator_child(ctx->alloc);
 
-    struct Iface* rawSocketIf = Er(SocketInterface_new(
-        socketFullPath->bytes, ctx->base, ctx->logger, ctx->tunAlloc));
+    struct Iface* rawSocketIf = NULL;
+    Err(Socket_connect(&rawSocketIf, socketFullPath->bytes, ctx->tunAlloc));
     struct SocketWrapper* sw = SocketWrapper_new(ctx->tunAlloc, ctx->logger);
     Iface_plumb(&sw->externalIf, rawSocketIf);
 
-    ctx->tunDevice = &sw->internalIf;
-    Iface_plumb(ctx->tunDevice, &ctx->nc->tunAdapt->tunIf);
+    ctx->tunIface = &sw->internalIf;
+    Iface_plumb(ctx->tunIface, &ctx->nc->tunAdapt->tunIf);
 
-    Er(SocketWrapper_addAddress(
+    Err(SocketWrapper_addAddress(
         &sw->externalIf, ctx->nc->myAddress->ip6.bytes, ctx->logger, ctx->alloc));
-    Er(SocketWrapper_setMTU(&sw->externalIf, DEFAULT_MTU, ctx->logger, ctx->alloc));
-    Er_ret();
+    Err(SocketWrapper_setMTU(&sw->externalIf, DEFAULT_MTU, ctx->logger, ctx->alloc));
+    return NULL;
 }
 
-static Er_DEFUN(void initTunnel2(String* desiredDeviceName,
+static Err_DEFUN initTunnel2(String* desiredDeviceName,
                         struct Context* ctx,
                         uint8_t addressPrefix,
-                        struct Allocator* errAlloc))
+                        struct Allocator* errAlloc)
 {
     Log_debug(ctx->logger, "Initializing TUN device [%s]",
               (desiredDeviceName) ? desiredDeviceName->bytes : "<auto>");
@@ -194,16 +206,22 @@ static Er_DEFUN(void initTunnel2(String* desiredDeviceName,
     char assignedTunName[TUNInterface_IFNAMSIZ];
     char* desiredName = (desiredDeviceName) ? desiredDeviceName->bytes : NULL;
 
-    if (ctx->tunDevice) {
-        Iface_unplumb(&ctx->nc->tunAdapt->tunIf, ctx->tunDevice);
+    if (ctx->tunIface) {
+        Iface_unplumb(&ctx->nc->tunAdapt->tunIf, ctx->tunIface);
         Allocator_free(ctx->tunAlloc);
-        ctx->tunDevice = NULL;
+        ctx->tunIface = NULL;
+        ctx->tun = NULL;
     }
     ctx->tunAlloc = Allocator_child(ctx->alloc);
-    ctx->tunDevice = Er(TUNInterface_new(
-        desiredName, assignedTunName, 0, ctx->base, ctx->logger, ctx->tunAlloc));
+    Err(TUNInterface_new(
+        &ctx->tun,
+        desiredName,
+        assignedTunName,
+        ctx->logger,
+        ctx->tunAlloc));
 
-    Iface_plumb(ctx->tunDevice, &ctx->nc->tunAdapt->tunIf);
+    ctx->tunIface = ctx->tun->iface;
+    Iface_plumb(ctx->tunIface, &ctx->nc->tunAdapt->tunIf);
 
     GlobalConfig_setTunName(ctx->globalConf, String_CONST(assignedTunName));
 
@@ -211,9 +229,8 @@ static Er_DEFUN(void initTunnel2(String* desiredDeviceName,
         Sockaddr_fromBytes(ctx->nc->myAddress->ip6.bytes, Sockaddr_AF_INET6, ctx->tunAlloc);
     myAddr->prefix = addressPrefix;
     myAddr->flags |= Sockaddr_flags_PREFIX;
-    Er(NetDev_addAddress(assignedTunName, myAddr, ctx->logger, errAlloc));
-    Er(NetDev_setMTU(assignedTunName, DEFAULT_MTU, ctx->logger, errAlloc));
-    Er_ret();
+    Err(NetDev_addAddress(assignedTunName, myAddr, ctx->logger, errAlloc));
+    return NetDev_setMTU(assignedTunName, DEFAULT_MTU, ctx->logger, errAlloc);
 }
 
 static void initTunfd(Dict* args, void* vcontext, String* txid, struct Allocator* requestAlloc)
@@ -230,44 +247,46 @@ static void initTunfd(Dict* args, void* vcontext, String* txid, struct Allocator
     int type = (tuntype) ? *tuntype : TUNMessageType_guess();
 
     struct Allocator* tunAlloc = Allocator_child(ctx->alloc);
-    struct Er_Ret* er = NULL;
-    struct Pipe* p = Er_check(&er, Pipe_forFd(fileno, false, ctx->base, ctx->logger, tunAlloc));
+    TUNInterface_t* tun = NULL;
+    RTypes_Error_t* er = TUNInterface_forFd(&tun, fileno, tunAlloc);
     if (er) {
-        Log_debug(ctx->logger, "Failed to create pipe [%s]", er->message);
+        const char* err = Rffi_printError(er, requestAlloc);
+        Log_debug(ctx->logger, "Failed to create pipe [%s]", err);
         String* error =
-            String_printf(requestAlloc, "Failed to configure tunnel [%s]", er->message);
+            String_printf(requestAlloc, "Failed to configure tunnel [%s]", err);
         sendResponse(error, ctx->admin, txid, requestAlloc);
         return;
     }
-    struct Iface* iface = NULL;
+    Iface_t* iface = tun->iface;
     if (type == TUNMessageType_NONE) {
-        struct AndroidWrapper* aw = AndroidWrapper_new(tunAlloc, ctx->logger);
-        Iface_plumb(&aw->externalIf, &p->iface);
-        iface = &aw->internalIf;
-    } else {
-        iface = &p->iface;
+        RTypes_IfWrapper_t aw = Rffi_android_create(tunAlloc);
+        Iface_plumb(aw.external, tun->iface);
+        iface = aw.internal;
     }
 
-    if (ctx->tunDevice) {
-        Iface_unplumb(&ctx->nc->tunAdapt->tunIf, ctx->tunDevice);
+    if (ctx->tunIface) {
+        Iface_unplumb(&ctx->nc->tunAdapt->tunIf, ctx->tunIface);
         Allocator_free(ctx->tunAlloc);
-        ctx->tunDevice = NULL;
+        ctx->tunIface = NULL;
+        ctx->tun = NULL;
     }
     Assert_true(!ctx->nc->tunAdapt->tunIf.connectedIf);
     ctx->tunAlloc = tunAlloc;
-    ctx->tunDevice = iface;
-    Iface_plumb(ctx->tunDevice, &ctx->nc->tunAdapt->tunIf);
+    ctx->tun = tun;
+    ctx->tunIface = iface;
+    Iface_plumb(iface, &ctx->nc->tunAdapt->tunIf);
 
     sendResponse(String_CONST("none"), ctx->admin, txid, requestAlloc);
 }
 
-static void stopTun(Dict* args, void* vcontext, String* txid, struct Allocator* requestAlloc)
+static void stopTun(Gcc_UNUSED Dict* args, void* vcontext, String* txid, struct Allocator* requestAlloc)
 {
     struct Context* ctx = Identity_check((struct Context*) vcontext);
-    if (ctx->tunDevice) {
-        Iface_unplumb(&ctx->nc->tunAdapt->tunIf, ctx->tunDevice);
+    if (ctx->tunIface) {
+        Iface_unplumb(&ctx->nc->tunAdapt->tunIf, ctx->tunIface);
         Allocator_free(ctx->tunAlloc);
-        ctx->tunDevice = NULL;
+        ctx->tunIface = NULL;
+        ctx->tun = NULL;
         sendResponse(String_new("none", requestAlloc), ctx->admin, txid, requestAlloc);
     } else {
         sendResponse(
@@ -278,14 +297,42 @@ static void stopTun(Dict* args, void* vcontext, String* txid, struct Allocator*
     }
 }
 
+static void tunWorkers(Dict* Gcc_UNUSED args, void* vcontext, String* txid, struct Allocator* requestAlloc)
+{
+    struct Context* ctx = Identity_check((struct Context*) vcontext);
+    if (ctx->tun) {
+        Object_t* workers = NULL;
+        RTypes_Error_t* err = TUNInterface_workerStates(&workers, ctx->tun, requestAlloc);
+        if (err) {
+            char* error = Rffi_printError(err, requestAlloc);
+            sendResponse(
+                String_new(error, requestAlloc),
+                ctx->admin,
+                txid,
+                requestAlloc);
+        } else {
+            Dict* output = Dict_new(requestAlloc);
+            Dict_putStringCC(output, "error", "none", requestAlloc);
+            Dict_putObject(output, String_CONST("workers"), workers, requestAlloc);
+            Admin_sendMessage(output, txid, ctx->admin);
+        }
+    } else {
+        sendResponse(
+            String_new("TUN device missing or does not use workers", requestAlloc),
+            ctx->admin,
+            txid,
+            requestAlloc);
+    }
+}
+
 static void initTunnel(Dict* args, void* vcontext, String* txid, struct Allocator* requestAlloc)
 {
     struct Context* const ctx = Identity_check((struct Context*) vcontext);
     String* desiredName = Dict_getStringC(args, "desiredTunName");
-    struct Er_Ret* er = NULL;
-    Er_check(&er, initTunnel2(desiredName, ctx, AddressCalc_ADDRESS_PREFIX_BITS, requestAlloc));
-    if (er) {
-        String* error = String_printf(requestAlloc, "Failed to configure tunnel [%s]", er->message);
+    RTypes_Error_t* err = initTunnel2(desiredName, ctx, AddressCalc_ADDRESS_PREFIX_BITS, requestAlloc);
+    if (err) {
+        String* error = String_printf(requestAlloc, "Failed to configure tunnel [%s]",
+            Rffi_printError(err, requestAlloc));
         sendResponse(error, ctx->admin, txid, requestAlloc);
         return;
     }
@@ -297,18 +344,17 @@ static void initSocket(Dict* args, void* vcontext, String* txid, struct Allocato
 {
     struct Context* const ctx = Identity_check((struct Context*) vcontext);
     String* socketFullPath = Dict_getStringC(args, "socketFullPath");
-    struct Er_Ret* er = NULL;
-    Er_check(&er, initSocket2(socketFullPath, ctx, AddressCalc_ADDRESS_PREFIX_BITS));
-    if (er) {
+    RTypes_Error_t* err = initSocket2(socketFullPath, ctx);
+    if (err) {
         String* error = String_printf(requestAlloc, "Failed to configure socket [%s]",
-            er->message);
+            Rffi_printError(err, requestAlloc));
         sendResponse(error, ctx->admin, txid, requestAlloc);
         return;
     }
     sendResponse(String_CONST("none"), ctx->admin, txid, requestAlloc);
 }
 
-static void nodeInfo(Dict* args, void* vcontext, String* txid, struct Allocator* requestAlloc)
+static void nodeInfo(Gcc_UNUSED Dict* args, void* vcontext, String* txid, struct Allocator* requestAlloc)
 {
     struct Context* const ctx = Identity_check((struct Context*) vcontext);
     String* myAddr = Address_toStringKey(ctx->nc->myAddress, requestAlloc);
@@ -328,14 +374,12 @@ static void nodeInfo(Dict* args, void* vcontext, String* txid, struct Allocator*
     Admin_sendMessage(out, txid, ctx->admin);
 }
 
-void Core_init(struct Allocator* alloc,
+Err_DEFUN Core_init(struct Allocator* alloc,
                struct Log* logger,
-               struct EventBase* eventBase,
+               EventBase_t* eventBase,
                uint8_t privateKey[32],
                struct Admin* admin,
                struct Random* rand,
-               struct Except* eh,
-               struct FakeNetwork* fakeNet,
                bool noSec)
 {
     struct Security* sec = NULL;
@@ -343,7 +387,7 @@ void Core_init(struct Allocator* alloc,
         sec = Security_new(alloc, logger, eventBase);
     }
     struct GlobalConfig* globalConf = GlobalConfig_new(alloc);
-    struct NetCore* nc = NetCore_new(privateKey, alloc, eventBase, rand, logger);
+    struct NetCore* nc = NetCore_new(privateKey, alloc, eventBase, rand, logger, !Defined(NOISE_NO));
 
     struct RouteGen* rg = RouteGen_new(alloc, logger);
 
@@ -356,7 +400,7 @@ void Core_init(struct Allocator* alloc,
 
     // The link between the Pathfinder and the core needs to be asynchronous.
     struct SubnodePathfinder* spf = SubnodePathfinder_new(
-        alloc, logger, eventBase, rand, nc->myAddress, privateKey, encodingScheme);
+        alloc, logger, eventBase, rand, nc->myAddress, privateKey, encodingScheme, nc->ca);
     struct ASynchronizer* spfAsync = ASynchronizer_new(alloc, eventBase, logger);
     Iface_plumb(&spfAsync->ifA, &spf->eventIf);
     EventEmitter_regPathfinderIface(nc->ee, &spfAsync->ifB);
@@ -371,12 +415,14 @@ void Core_init(struct Allocator* alloc,
     SubnodePathfinder_start(spf);
 
     // ------------------- Register RPC functions ----------------------- //
+    SubnodePathfinder_admin_register(spf, admin, alloc);
+    PeeringSeeder_admin_register(spf->ps, admin, alloc);
     UpperDistributor_admin_register(nc->upper, admin, alloc);
     RouteGen_admin_register(rg, admin, alloc);
     InterfaceController_admin_register(nc->ifController, admin, alloc);
-    SwitchPinger_admin_register(nc->sp, admin, alloc);
+    SwitchPinger_admin_register(spf->sp, admin, alloc);
     UDPInterface_admin_register(
-        eventBase, alloc, logger, admin, nc->ifController, fakeNet, globalConf);
+        eventBase, alloc, logger, admin, nc->ifController, globalConf);
 #ifdef HAS_ETH_INTERFACE
     ETHInterface_admin_register(eventBase, alloc, logger, admin, nc->ifController);
 #endif
@@ -385,7 +431,7 @@ void Core_init(struct Allocator* alloc,
     ReachabilityCollector_admin_register(spf->rc, admin, alloc);
 
     AuthorizedPasswords_init(admin, nc->ca, alloc);
-    Admin_registerFunction("ping", adminPing, admin, false, NULL, admin);
+    Admin_registerFunctionNoArgs("ping", adminPing, admin, false, admin);
     if (!noSec) {
         Security_admin_register(alloc, logger, sec, admin);
     }
@@ -405,9 +451,9 @@ void Core_init(struct Allocator* alloc,
     ctx->encodingScheme = encodingScheme;
     ctx->globalConf = globalConf;
 
-    Admin_registerFunction("Core_exit", adminExit, ctx, true, NULL, admin);
+    Admin_registerFunctionNoArgs("Core_exit", adminExit, ctx, true, admin);
 
-    Admin_registerFunction("Core_pid", adminPid, admin, false, NULL, admin);
+    Admin_registerFunctionNoArgs("Core_pid", adminPid, admin, false, admin);
 
     Admin_registerFunction("Core_initTunnel", initTunnel, ctx, true,
         ((struct Admin_FunctionArg[]) {
@@ -420,48 +466,56 @@ void Core_init(struct Allocator* alloc,
             { .name = "type", .required = 0, .type = "Int" }
         }), admin);
 
-    Admin_registerFunction("Core_stopTun", stopTun, ctx, true, NULL, admin);
+    Admin_registerFunctionNoArgs("Core_stopTun", stopTun, ctx, true, admin);
 
-    Admin_registerFunction("Core_nodeInfo", nodeInfo, ctx, false, NULL, admin);
+    Admin_registerFunctionNoArgs("Core_nodeInfo", nodeInfo, ctx, false, admin);
 
     Admin_registerFunction("Core_initSocket", initSocket, ctx, true,
         ((struct Admin_FunctionArg[]) {
             { .name = "socketFullPath", .required = 1, .type = "String" },
             { .name = "socketAttemptToCreate", .required = 0, .type = "Int" }
         }), admin);
+
+    Admin_registerFunctionNoArgs("Core_tunWorkers", tunWorkers, ctx, true, admin);
+    return NULL;
 }
 
 int Core_main(int argc, char** argv)
 {
-    struct Except* eh = NULL;
+    Glock_init();
 
     if (argc != 3) {
-        Except_throw(eh, "This is internal to cjdns and shouldn't started manually.");
+        Assert_failure( "This is internal to cjdns and shouldn't started manually.");
     }
 
-    struct Allocator* alloc = MallocAllocator_new(ALLOCATOR_FAILSAFE);
+    struct Allocator* alloc = Allocator_new(ALLOCATOR_FAILSAFE);
     struct Log* preLogger = FileWriterLog_new(stderr, alloc);
-    struct EventBase* eventBase = EventBase_new(alloc);
+    Rffi_setLogger(preLogger);
+    EventBase_t* eventBase = EventBase_new(alloc);
 
     // -------------------- Setup the Pre-Logger ---------------------- //
     struct Log* logger = IndirectLog_new(alloc);
     IndirectLog_set(logger, preLogger);
 
     // -------------------- Setup the PRNG ---------------------- //
-    struct Random* rand = NanotimeEntropyProvider_newDefaultRandom(eventBase, logger, eh, alloc);
+    struct Random* rand = NULL;
+    Err_assert(NanotimeEntropyProvider_newDefaultRandom(&rand, eventBase, logger, alloc));
 
     // -------------------- Change Canary Value ---------------------- //
     Allocator_setCanary(alloc, (uintptr_t)Random_uint64(rand));
 
     struct Allocator* tempAlloc = Allocator_child(alloc);
     // Not using tempalloc because we're going to keep this pipe around for admin
-    struct PipeServer* clientPipe = PipeServer_named(argv[2], eventBase, eh, logger, alloc);
+    Socket_Server_t* ss = NULL;
+    Err_assert(Socket_server(&ss, argv[2], alloc));
     Log_debug(logger, "Getting pre-configuration from client");
-    struct Message* preConf =
-        InterfaceWaiter_waitForData(&clientPipe->iface.iface, eventBase, tempAlloc, eh);
+    Message_t* preConf = NULL;
+    Err_assert(InterfaceWaiter_waitForData(&preConf, ss->iface, eventBase, tempAlloc));
     Log_debug(logger, "Finished getting pre-configuration from client");
-    struct Sockaddr* addr = Sockaddr_clone(Er_assert(AddrIface_popAddr(preConf)), tempAlloc);
-    Dict* config = Except_er(eh, BencMessageReader_read(preConf, tempAlloc));
+    struct Sockaddr_storage addr;
+    Err_assert(AddrIface_popAddr(&addr, preConf));
+    Dict* config = NULL;
+    Err_assert(BencMessageReader_read(&config, preConf, tempAlloc));
 
     String* privateKeyHex = Dict_getStringC(config, "privateKey");
     Dict* adminConf = Dict_getDictC(config, "admin");
@@ -469,37 +523,37 @@ int Core_main(int argc, char** argv)
     String* bind = Dict_getStringC(adminConf, "bind");
     if (!(pass && privateKeyHex && bind)) {
         if (!pass) {
-            Except_throw(eh, "Expected 'pass'");
+            Assert_failure( "Expected 'pass'");
         }
         if (!bind) {
-            Except_throw(eh, "Expected 'bind'");
+            Assert_failure("Expected 'bind'");
         }
         if (!privateKeyHex) {
-            Except_throw(eh, "Expected 'privateKey'");
+            Assert_failure("Expected 'privateKey'");
         }
-        Except_throw(eh, "Expected 'pass', 'privateKey' and 'bind' in configuration.");
+        Assert_failure("Expected 'pass', 'privateKey' and 'bind' in configuration.");
     }
     Log_keys(logger, "Starting core with admin password [%s]", pass->bytes);
     uint8_t privateKey[32];
     if (privateKeyHex->len != 64
         || Hex_decode(privateKey, 32, (uint8_t*) privateKeyHex->bytes, 64) != 32)
     {
-        Except_throw(eh, "privateKey must be 64 bytes of hex.");
+        Assert_failure("privateKey must be 64 bytes of hex.");
     }
 
     struct Sockaddr_storage bindAddr;
     if (Sockaddr_parse(bind->bytes, &bindAddr)) {
-        Except_throw(eh, "bind address [%s] unparsable", bind->bytes);
+        Assert_failure("bind address [%s] unparsable", bind->bytes);
     }
 
     // --------------------- Bind Admin UDP --------------------- //
-    struct UDPAddrIface* udpAdmin =
-        Except_er(eh, UDPAddrIface_new(eventBase, &bindAddr.addr, alloc, logger));
+    struct UDPAddrIface* udpAdmin = NULL;
+    Err_assert(UDPAddrIface_new(&udpAdmin, &bindAddr.addr, alloc));
 
     // ---- Setup a muxer so we can get admin from socket or UDP ---- //
     struct AddrIfaceMuxer* muxer = AddrIfaceMuxer_new(logger, alloc);
-    Iface_plumb(&udpAdmin->generic.iface, AddrIfaceMuxer_registerIface(muxer, alloc));
-    Iface_plumb(&clientPipe->iface.iface, AddrIfaceMuxer_registerIface(muxer, alloc));
+    Iface_plumb(udpAdmin->generic.iface, AddrIfaceMuxer_registerIface(muxer, alloc));
+    Iface_plumb(ss->iface, AddrIfaceMuxer_registerIface(muxer, alloc));
 
     // --------------------- Setup Admin --------------------- //
     struct Admin* admin = Admin_new(&muxer->iface, logger, eventBase, pass);
@@ -513,6 +567,7 @@ int Core_main(int argc, char** argv)
         struct Log* adminLogger = AdminLog_registerNew(admin, alloc, rand, eventBase);
         IndirectLog_set(logger, adminLogger);
         logger = adminLogger;
+        Rffi_setLogger(logger);
     }
 
     // --------------------- Inform client of UDP Addr --------------------- //
@@ -525,14 +580,14 @@ int Core_main(int argc, char** argv)
         String_CONST("admin"), Dict_OBJ(&adminResponse), NULL
     ));
     // This always times out because the angel doesn't respond.
-    struct Message* clientResponse = Message_new(0, 512, tempAlloc);
-    Er_assert(BencMessageWriter_write(&response, clientResponse));
-    Er_assert(AddrIface_pushAddr(clientResponse, addr));
-    Iface_CALL(clientPipe->iface.iface.send, clientResponse, &clientPipe->iface.iface);
+    Message_t* clientResponse = Message_new(0, 512, tempAlloc);
+    Err_assert(BencMessageWriter_write(&response, clientResponse));
+    Err_assert(AddrIface_pushAddr(clientResponse, &addr.addr));
+    Iface_CALL(ss->iface->send, clientResponse, ss->iface);
 
     Allocator_free(tempAlloc);
 
-    Core_init(alloc, logger, eventBase, privateKey, admin, rand, eh, NULL, false);
+    Err_assert(Core_init(alloc, logger, eventBase, privateKey, admin, rand, false));
     EventBase_beginLoop(eventBase);
     return 0;
 }
