@@ -24,13 +24,14 @@ Linker_require("crypto/random/nanotime/NanotimeEntropyProvider.c")
 struct NanotimeEntropyProvider;
 
 void NanotimeEntropyProvider_start(struct Random* provideTo,
-                                struct EventBase* base,
+                                EventBase_t* base,
                                 struct Log* logger,
                                 struct Allocator* alloc);
 
-struct Random* NanotimeEntropyProvider_newDefaultRandom(struct EventBase* base,
-                                                     struct Log* logger,
-                                                     struct Except* eh,
-                                                     struct Allocator* alloc);
+Err_DEFUN NanotimeEntropyProvider_newDefaultRandom(
+    struct Random** out,
+    EventBase_t* base,
+    struct Log* logger,
+    struct Allocator* alloc);
 
 #endif
