@@ -14,14 +14,15 @@
  */
 #include "crypto/random/Random.h"
 #include "memory/Allocator.h"
-#include "memory/MallocAllocator.h"
 
 #include <unistd.h>
 
-int main()
+int randombytes_main(void);
+int randombytes_main()
 {
-    struct Allocator* alloc = MallocAllocator_new(1<<20);
-    struct Random* rand = Random_new(alloc, NULL, NULL);
+    struct Allocator* alloc = Allocator_new(1<<20);
+    struct Random* rand = NULL;
+    Err_assert(Random_new(&rand, alloc, NULL));
 
     struct {
         uint64_t align;
