@@ -20,6 +20,7 @@
 #include "util/Linker.h"
 
 Linker_require("crypto/random/seed/BsdKernArndSysctlRandomSeed.c")
-struct RandomSeed* BsdKernArndSysctlRandomSeed_new(struct Allocator* alloc);
+RandomSeed_t* BsdKernArndSysctlRandomSeed_new(struct Allocator* alloc);
+#define BsdKernArndSysctlRandomSeed_EXISTS
 
 #endif
