@@ -20,6 +20,7 @@
 #include "util/Linker.h"
 
 Linker_require("crypto/random/seed/DevUrandomRandomSeed.c")
-struct RandomSeed* DevUrandomRandomSeed_new(struct Allocator* alloc);
+RandomSeed_t* DevUrandomRandomSeed_new(struct Allocator* alloc);
+#define DevUrandomRandomSeed_EXISTS
 
 #endif
