@@ -20,6 +20,7 @@
 #include "util/Linker.h"
 
 Linker_require("crypto/random/seed/GetEntropyRandomSeed.c")
-struct RandomSeed* GetEntropyRandomSeed_new(struct Allocator* alloc);
+RandomSeed_t* GetEntropyRandomSeed_new(struct Allocator* alloc);
+#define GetEntropyRandomSeed_EXISTS
 
 #endif
