@@ -16,7 +16,6 @@
 #define SystemRandomSeed_H
 
 #include "crypto/random/seed/RandomSeed.h"
-#include "crypto/random/seed/SystemRandomSeed.h"
 #include "util/Linker.h"
 Linker_require("crypto/random/seed/SystemRandomSeed.c")
 
@@ -24,7 +23,7 @@ Linker_require("crypto/random/seed/SystemRandomSeed.c")
  * Create a new randomseed with the default system providers as well as optionally more providers
  * additionalProviders is a null terminalted list.
  */
-struct RandomSeed* SystemRandomSeed_new(RandomSeed_Provider* additionalProviders,
+RandomSeed_t* SystemRandomSeed_new(RandomSeed_Provider* additionalProviders,
                                         int additionalProviderCount,
                                         struct Log* logger,
                                         struct Allocator* alloc);
