@@ -57,6 +57,8 @@ struct SwitchPinger_pvt
     // If it's an rpath message
     uint64_t rpath;
 
+    struct Control_LlAddr lladdrMsg;
+
     /** The error code if an error has been returned (see Error.h) */
     int error;
 
@@ -82,66 +84,67 @@ struct Ping
 };
 
 // incoming message from network, pointing to the beginning of the switch header.
-static Iface_DEFUN messageFromControlHandler(struct Message* msg, struct Iface* iface)
+static Iface_DEFUN messageFromControlHandler(Message_t* msg, struct Iface* iface)
 {
     struct SwitchPinger_pvt* ctx = Identity_check((struct SwitchPinger_pvt*) iface);
     struct RouteHeader rh;
-    Er_assert(Message_epop(msg, &rh, RouteHeader_SIZE));
+    Err(Message_epop(msg, &rh, RouteHeader_SIZE));
     ctx->incomingLabel = Endian_bigEndianToHost64(rh.sh.label_be);
     ctx->incomingVersion = 0;
     Bits_memset(&ctx->incomingSnodeAddr, 0, sizeof ctx->incomingSnodeAddr);
     Bits_memset(ctx->incomingKey, 0, sizeof ctx->incomingKey);
     ctx->incomingSnodeKbps = 0;
     ctx->rpath = 0;
+    Bits_memset(&ctx->lladdrMsg, 0, sizeof ctx->lladdrMsg);
 
-    struct Control* ctrl = (struct Control*) msg->bytes;
+    struct Control* ctrl = (struct Control*) Message_bytes(msg);
     if (ctrl->header.type_be == Control_PONG_be) {
-        Er_assert(Message_eshift(msg, -Control_Header_SIZE));
+        Err(Message_eshift(msg, -Control_Header_SIZE));
         ctx->error = Error_NONE;
-        if (msg->length >= Control_Pong_MIN_SIZE) {
-            struct Control_Ping* pongHeader = (struct Control_Ping*) msg->bytes;
+        if (Message_getLength(msg) >= Control_Pong_MIN_SIZE) {
+            struct Control_Ping* pongHeader = (struct Control_Ping*) Message_bytes(msg);
             ctx->incomingVersion = Endian_bigEndianToHost32(pongHeader->version_be);
             if (pongHeader->magic != Control_Pong_MAGIC) {
                 Log_debug(ctx->logger, "dropped invalid switch pong");
-                return NULL;
+                return Error(msg, "INVALID");
             }
-            Er_assert(Message_eshift(msg, -Control_Pong_HEADER_SIZE));
+            Err(Message_eshift(msg, -Control_Pong_HEADER_SIZE));
         } else {
-            Log_debug(ctx->logger, "got runt pong message, length: [%d]", msg->length);
-            return NULL;
+            Log_debug(ctx->logger, "got runt pong message, length: [%d]", Message_getLength(msg));
+            return Error(msg, "RUNT");
         }
 
     } else if (ctrl->header.type_be == Control_KEYPONG_be) {
-        Er_assert(Message_eshift(msg, -Control_Header_SIZE));
+        Err(Message_eshift(msg, -Control_Header_SIZE));
         ctx->error = Error_NONE;
-        if (msg->length >= Control_KeyPong_HEADER_SIZE && msg->length <= Control_KeyPong_MAX_SIZE) {
-            struct Control_KeyPing* pongHeader = (struct Control_KeyPing*) msg->bytes;
+        if (Message_getLength(msg) >= Control_KeyPong_HEADER_SIZE && Message_getLength(msg) <= Control_KeyPong_MAX_SIZE) {
+            struct Control_KeyPing* pongHeader = (struct Control_KeyPing*) Message_bytes(msg);
             ctx->incomingVersion = Endian_bigEndianToHost32(pongHeader->version_be);
             if (pongHeader->magic != Control_KeyPong_MAGIC) {
                 Log_debug(ctx->logger, "dropped invalid switch key-pong");
-                return NULL;
+                return Error(msg, "INVALID");
             }
             Bits_memcpy(ctx->incomingKey, pongHeader->key, 32);
-            Er_assert(Message_eshift(msg, -Control_KeyPong_HEADER_SIZE));
-        } else if (msg->length > Control_KeyPong_MAX_SIZE) {
-            Log_debug(ctx->logger, "got overlong key-pong message, length: [%d]", msg->length);
-            return NULL;
+            Err(Message_eshift(msg, -Control_KeyPong_HEADER_SIZE));
+        } else if (Message_getLength(msg) > Control_KeyPong_MAX_SIZE) {
+            Log_debug(ctx->logger, "got overlong key-pong message, length: [%d]", Message_getLength(msg));
+            return Error(msg, "INVALID");
         } else {
-            Log_debug(ctx->logger, "got runt key-pong message, length: [%d]", msg->length);
-            return NULL;
+            Log_debug(ctx->logger, "got runt key-pong message, length: [%d]", Message_getLength(msg));
+            return Error(msg, "RUNT");
         }
 
     } else if (ctrl->header.type_be == Control_GETSNODE_REPLY_be) {
-        Er_assert(Message_eshift(msg, -Control_Header_SIZE));
+        Err(Message_eshift(msg, -Control_Header_SIZE));
         ctx->error = Error_NONE;
-        if (msg->length < Control_GetSnode_HEADER_SIZE) {
-            Log_debug(ctx->logger, "got runt GetSnode message, length: [%d]", msg->length);
-            return NULL;
+        if (Message_getLength(msg) < Control_GetSnode_HEADER_SIZE) {
+            Log_debug(ctx->logger, "got runt GetSnode message, length: [%d]", Message_getLength(msg));
+            return Error(msg, "RUNT");
         }
-        struct Control_GetSnode* hdr = (struct Control_GetSnode*) msg->bytes;
+        struct Control_GetSnode* hdr = (struct Control_GetSnode*) Message_bytes(msg);
         if (hdr->magic != Control_GETSNODE_REPLY_MAGIC) {
             Log_debug(ctx->logger, "dropped invalid GetSnode");
-            return NULL;
+            return Error(msg, "INVALID");
         }
         if (Bits_isZero(hdr->snodeKey, 32)) {
             Log_debug(ctx->logger, "Peer doesn't have an snode");
@@ -149,7 +152,7 @@ static Iface_DEFUN messageFromControlHandler(struct Message* msg, struct Iface*
         }
         if (!AddressCalc_addressForPublicKey(ctx->incomingSnodeAddr.ip6.bytes, hdr->snodeKey)) {
             Log_debug(ctx->logger, "dropped invalid GetSnode key");
-            return NULL;
+            return Error(msg, "INVALID");
         }
         ctx->incomingVersion = Endian_hostToBigEndian32(hdr->version_be);
         Bits_memcpy(ctx->incomingSnodeAddr.key, hdr->snodeKey, 32);
@@ -158,43 +161,45 @@ static Iface_DEFUN messageFromControlHandler(struct Message* msg, struct Iface*
         ctx->incomingSnodeAddr.path = Endian_bigEndianToHost64(pathToSnode_be);
         ctx->incomingSnodeAddr.protocolVersion = Endian_bigEndianToHost32(hdr->snodeVersion_be);
         ctx->incomingSnodeKbps = Endian_bigEndianToHost32(hdr->kbps_be);
-        Er_assert(Message_eshift(msg, -Control_GetSnode_HEADER_SIZE));
+        Err(Message_eshift(msg, -Control_GetSnode_HEADER_SIZE));
 
     } else if (ctrl->header.type_be == Control_RPATH_REPLY_be) {
-        Er_assert(Message_eshift(msg, -Control_Header_SIZE));
+        Err(Message_eshift(msg, -Control_Header_SIZE));
         ctx->error = Error_NONE;
-        if (msg->length < Control_RPath_HEADER_SIZE) {
-            Log_debug(ctx->logger, "got runt RPath message, length: [%d]", msg->length);
-            return NULL;
+        if (Message_getLength(msg) < Control_RPath_HEADER_SIZE) {
+            Log_debug(ctx->logger, "got runt RPath message, length: [%d]", Message_getLength(msg));
+            return Error(msg, "RUNT");
         }
-        struct Control_RPath* hdr = (struct Control_RPath*) msg->bytes;
+        struct Control_RPath* hdr = (struct Control_RPath*) Message_bytes(msg);
         if (hdr->magic != Control_RPATH_REPLY_MAGIC) {
             Log_debug(ctx->logger, "dropped invalid RPATH (bad magic)");
-            return NULL;
+            return Error(msg, "INVALID");
         }
         ctx->incomingVersion = Endian_hostToBigEndian32(hdr->version_be);
         uint64_t rpath_be;
         Bits_memcpy(&rpath_be, hdr->rpath_be, 8);
         ctx->rpath = Endian_bigEndianToHost64(rpath_be);
-        Er_assert(Message_eshift(msg, -Control_RPath_HEADER_SIZE));
+        Err(Message_eshift(msg, -Control_RPath_HEADER_SIZE));
 
     } else if (ctrl->header.type_be == Control_ERROR_be) {
-        Er_assert(Message_eshift(msg, -Control_Header_SIZE));
-        Assert_true((uint8_t*)&ctrl->content.error.errorType_be == msg->bytes);
-        if (msg->length < (Control_Error_HEADER_SIZE + SwitchHeader_SIZE + Control_Header_SIZE)) {
+        Err(Message_eshift(msg, -Control_Header_SIZE));
+        Assert_true((uint8_t*)&ctrl->content.error.errorType_be == Message_bytes(msg));
+        if (Message_getLength(msg) < (Control_Error_HEADER_SIZE + SwitchHeader_SIZE + Control_Header_SIZE)) {
             Log_debug(ctx->logger, "runt error packet");
-            return NULL;
+            return Error(msg, "RUNT");
         }
 
-        ctx->error = Er_assert(Message_epop32be(msg));
-        Er_assert(Message_epush32be(msg, 0));
+        uint32_t err = 0;
+        Err(Message_epop32be(&err, msg));
+        ctx->error = err;
+        Err(Message_epush32be(msg, 0));
 
-        Er_assert(Message_eshift(msg, -(Control_Error_HEADER_SIZE + SwitchHeader_SIZE)));
+        Err(Message_eshift(msg, -(Control_Error_HEADER_SIZE + SwitchHeader_SIZE)));
 
-        struct Control* origCtrl = (struct Control*) msg->bytes;
+        struct Control* origCtrl = (struct Control*) Message_bytes(msg);
 
-        Log_debug(ctx->logger, "error [%s] was caused by our [%s]",
-                  Error_strerror(ctx->error),
+        Log_debug(ctx->logger, "error [%d] was caused by our [%s]",
+                  ctx->error,
                   Control_typeString(origCtrl->header.type_be));
 
         int shift;
@@ -205,17 +210,31 @@ static Iface_DEFUN messageFromControlHandler(struct Message* msg, struct Iface*
         } else {
             Assert_failure("problem in Ducttape.c");
         }
-        if (msg->length < -shift) {
+        if (Message_getLength(msg) < -shift) {
             Log_debug(ctx->logger, "runt error packet");
         }
-        Er_assert(Message_eshift(msg, shift));
+        Err(Message_eshift(msg, shift));
+
+    } else if (ctrl->header.type_be == Control_LlAddr_REPLY_be) {
+        Err(Message_eshift(msg, -Control_Header_SIZE));
+        ctx->error = Error_NONE;
+
+        if (Message_getLength(msg) < Control_LlAddr_HEADER_SIZE) {
+            Log_debug(ctx->logger, "got runt LlAddr message, length: [%d]", Message_getLength(msg));
+            return Error(msg, "RUNT");
+        }
+        Err(Message_epop(msg, &ctx->lladdrMsg, sizeof ctx->lladdrMsg));
+        if (ctx->lladdrMsg.magic != Control_LlAddr_REPLY_MAGIC) {
+            Log_debug(ctx->logger, "dropped invalid LLADDR reply (bad magic)");
+            return Error(msg, "INVALID");
+        }
 
     } else {
         // If it gets here then Ducttape.c is failing.
         Assert_true(false);
     }
 
-    String* msgStr = &(String) { .bytes = (char*) msg->bytes, .len = msg->length };
+    String* msgStr = &(String) { .bytes = (char*) Message_bytes(msg), .len = Message_getLength(msg) };
     Pinger_pongReceived(msgStr, ctx->pinger);
     Bits_memset(ctx->incomingKey, 0, 32);
     return NULL;
@@ -244,17 +263,20 @@ static void onPingResponse(String* data, uint32_t milliseconds, void* vping)
     uint32_t version = p->context->incomingVersion;
     struct SwitchPinger_Response* resp =
         Allocator_calloc(p->pub.pingAlloc, sizeof(struct SwitchPinger_Response), 1);
-    resp->version = p->context->incomingVersion;
     resp->res = err;
     resp->label = label;
-    resp->data = data;
     resp->milliseconds = milliseconds;
-    resp->version = version;
-    Bits_memcpy(resp->key, p->context->incomingKey, 32);
-    Bits_memcpy(&resp->snode, &p->context->incomingSnodeAddr, sizeof(struct Address));
-    resp->kbpsLimit = p->context->incomingSnodeKbps;
-    resp->rpath = p->context->rpath;
     resp->ping = &p->pub;
+    if (err != SwitchPinger_Result_TIMEOUT) {
+        resp->version = p->context->incomingVersion;
+        resp->data = data;
+        resp->version = version;
+        Bits_memcpy(resp->key, p->context->incomingKey, 32);
+        Bits_memcpy(&resp->snode, &p->context->incomingSnodeAddr, sizeof(struct Address));
+        resp->kbpsLimit = p->context->incomingSnodeKbps;
+        resp->rpath = p->context->rpath;
+        Bits_memcpy(&resp->lladdr, &p->context->lladdrMsg, sizeof p->context->lladdrMsg);
+    }
     p->onResponse(resp, p->pub.onResponseContext);
 }
 
@@ -262,32 +284,32 @@ static void sendPing(String* data, void* sendPingContext)
 {
     struct Ping* p = Identity_check((struct Ping*) sendPingContext);
 
-    struct Message* msg = Message_new(0, data->len + 512, p->pub.pingAlloc);
+    Message_t* msg = Message_new(0, data->len + 512, p->pub.pingAlloc);
 
-    while (((uintptr_t)msg->bytes - data->len) % 4) {
-        Er_assert(Message_epush8(msg, 0));
+    while (((uintptr_t)Message_bytes(msg) - data->len) % 4) {
+        Err_assert(Message_epush8(msg, 0));
     }
-    msg->length = 0;
+    Err_assert(Message_truncate(msg, 0));
 
-    Er_assert(Message_epush(msg, data->bytes, data->len));
-    Assert_true(!((uintptr_t)msg->bytes % 4) && "alignment fault");
+    Err_assert(Message_epush(msg, data->bytes, data->len));
+    Assert_true(!((uintptr_t)Message_bytes(msg) % 4) && "alignment fault");
 
     if (p->pub.type == SwitchPinger_Type_KEYPING) {
-        Er_assert(Message_epush(msg, NULL, Control_KeyPing_HEADER_SIZE));
-        struct Control_KeyPing* keyPingHeader = (struct Control_KeyPing*) msg->bytes;
+        Err_assert(Message_epush(msg, NULL, Control_KeyPing_HEADER_SIZE));
+        struct Control_KeyPing* keyPingHeader = (struct Control_KeyPing*) Message_bytes(msg);
         keyPingHeader->magic = Control_KeyPing_MAGIC;
         keyPingHeader->version_be = Endian_hostToBigEndian32(Version_CURRENT_PROTOCOL);
         Bits_memcpy(keyPingHeader->key, p->context->myAddr->key, 32);
 
     } else if (p->pub.type == SwitchPinger_Type_PING) {
-        Er_assert(Message_epush(msg, NULL, Control_Ping_HEADER_SIZE));
-        struct Control_Ping* pingHeader = (struct Control_Ping*) msg->bytes;
+        Err_assert(Message_epush(msg, NULL, Control_Ping_HEADER_SIZE));
+        struct Control_Ping* pingHeader = (struct Control_Ping*) Message_bytes(msg);
         pingHeader->magic = Control_Ping_MAGIC;
         pingHeader->version_be = Endian_hostToBigEndian32(Version_CURRENT_PROTOCOL);
 
     } else if (p->pub.type == SwitchPinger_Type_GETSNODE) {
-        Er_assert(Message_epush(msg, NULL, Control_GetSnode_HEADER_SIZE));
-        struct Control_GetSnode* hdr = (struct Control_GetSnode*) msg->bytes;
+        Err_assert(Message_epush(msg, NULL, Control_GetSnode_HEADER_SIZE));
+        struct Control_GetSnode* hdr = (struct Control_GetSnode*) Message_bytes(msg);
         hdr->magic = Control_GETSNODE_QUERY_MAGIC;
         hdr->version_be = Endian_hostToBigEndian32(Version_CURRENT_PROTOCOL);
         hdr->kbps_be = Endian_hostToBigEndian32(p->pub.kbpsLimit);
@@ -297,19 +319,28 @@ static void sendPing(String* data, void* sendPingContext)
         hdr->snodeVersion_be = Endian_hostToBigEndian32(p->pub.snode.protocolVersion);
 
     } else if (p->pub.type == SwitchPinger_Type_RPATH) {
-        Er_assert(Message_epush(msg, NULL, Control_RPath_HEADER_SIZE));
-        struct Control_RPath* hdr = (struct Control_RPath*) msg->bytes;
+        Err_assert(Message_epush(msg, NULL, Control_RPath_HEADER_SIZE));
+        struct Control_RPath* hdr = (struct Control_RPath*) Message_bytes(msg);
         hdr->magic = Control_RPATH_QUERY_MAGIC;
         hdr->version_be = Endian_hostToBigEndian32(Version_CURRENT_PROTOCOL);
         uint64_t path_be = Endian_hostToBigEndian64(p->label);
         Bits_memcpy(hdr->rpath_be, &path_be, 8);
 
+    } else if (p->pub.type == SwitchPinger_Type_LLADDR) {
+        struct Control_LlAddr addr = {
+            .magic = Control_LlAddr_QUERY_MAGIC,
+            .version_be = Endian_hostToBigEndian32(Version_CURRENT_PROTOCOL),
+            // Lazy
+            .addr.payload = { .type = 0, .len = 2, }
+        };
+        Err_assert(Message_epush(msg, &addr, sizeof addr));
+
     } else {
         Assert_failure("unexpected ping type");
     }
 
-    Er_assert(Message_eshift(msg, Control_Header_SIZE));
-    struct Control* ctrl = (struct Control*) msg->bytes;
+    Err_assert(Message_eshift(msg, Control_Header_SIZE));
+    struct Control* ctrl = (struct Control*) Message_bytes(msg);
     ctrl->header.checksum_be = 0;
     if (p->pub.type == SwitchPinger_Type_PING) {
         ctrl->header.type_be = Control_PING_be;
@@ -319,10 +350,12 @@ static void sendPing(String* data, void* sendPingContext)
         ctrl->header.type_be = Control_GETSNODE_QUERY_be;
     } else if (p->pub.type == SwitchPinger_Type_RPATH) {
         ctrl->header.type_be = Control_RPATH_QUERY_be;
+    } else if (p->pub.type == SwitchPinger_Type_LLADDR) {
+        ctrl->header.type_be = Control_LlAddr_QUERY_be;
     } else {
         Assert_failure("unexpected type");
     }
-    ctrl->header.checksum_be = Checksum_engine_be(msg->bytes, msg->length);
+    ctrl->header.checksum_be = Checksum_engine_be(Message_bytes(msg), Message_getLength(msg));
 
     struct RouteHeader rh;
     Bits_memset(&rh, 0, RouteHeader_SIZE);
@@ -330,7 +363,7 @@ static void sendPing(String* data, void* sendPingContext)
     rh.sh.label_be = Endian_hostToBigEndian64(p->label);
     SwitchHeader_setVersion(&rh.sh, SwitchHeader_CURRENT_VERSION);
 
-    Er_assert(Message_epush(msg, &rh, RouteHeader_SIZE));
+    Err_assert(Message_epush(msg, &rh, RouteHeader_SIZE));
 
     Iface_send(&p->context->pub.controlHandlerIf, msg);
 }
@@ -369,13 +402,12 @@ String* SwitchPinger_resultString(enum SwitchPinger_Result result)
     };
 }
 
-static int onPingFree(struct Allocator_OnFreeJob* job)
+static void onPingFree(struct Allocator_OnFreeJob* job)
 {
     struct Ping* ping = Identity_check((struct Ping*)job->userData);
     struct SwitchPinger_pvt* ctx = Identity_check(ping->context);
     ctx->outstandingPings--;
     Assert_true(ctx->outstandingPings >= 0);
-    return 0;
 }
 
 struct SwitchPinger_Ping* SwitchPinger_newPing(uint64_t label,
@@ -417,7 +449,7 @@ struct SwitchPinger_Ping* SwitchPinger_newPing(uint64_t label,
     return &ping->pub;
 }
 
-struct SwitchPinger* SwitchPinger_new(struct EventBase* eventBase,
+struct SwitchPinger* SwitchPinger_new(EventBase_t* eventBase,
                                       struct Random* rand,
                                       struct Log* logger,
                                       struct Address* myAddr,
