@@ -21,6 +21,8 @@ const nThen = require('nthen');
 const Crypto = require('crypto');
 const Saferphore = require('saferphore');
 
+var seed = process.env.SOURCE_DATE_EPOCH || Crypto.randomBytes(32).toString('hex');
+
 /*::
 export type Builder_File_t = {|
     includes: string[],
@@ -39,8 +41,6 @@ export type Builder_Compiler_t = {|
     version: string
 |};
 export type Builder_State_t = {|
-    rebuildIfChanges: string[],
-    rebuildIfChangesHash: string,
     compilerType: Builder_Compiler_t,
     cFiles: { [string]: Builder_File_t },
     hFiles: { [string]: Builder_Hfile_t },
@@ -49,13 +49,15 @@ export type Builder_Linter_t = (string, string, (string, bool)=>void)=>void;
 export type Builder_TestRunnerCb_t = (string, bool)=>void;
 export type Builder_t = {|
     cc: (string[], (number, string, string)=>void)=>void,
+    buildLibrary: (string)=>void,
     buildExecutable: (string, ?string)=>void,
     buildTest: (string)=>string,
     runTest: (string, (string, Builder_TestRunnerCb_t)=>void)=>void,
     lintFiles: (Builder_Linter_t)=>void,
     config: Builder_Config_t,
     tmpFile: (?string)=>string,
-    compilerType: () => Builder_Compiler_t
+    compilerType: () => Builder_Compiler_t,
+    fileCflags: {[string]: string[]},
 |};
 export type Builder_BaseConfig_t = {|
     systemName?: ?string,
@@ -71,12 +73,23 @@ export type Builder_Config_t = {
     ldflags: string[],
     libs: string[],
     jobs: number,
-    fileCflags: {[string]: string[]},
 } & {[string]:any};
 import type { Nthen_WaitFor_t } from 'nthen';
 import type { Saferphore_t } from 'saferphore';
 export type Builder_Stage_t = (Builder_t, Nthen_WaitFor_t)=>void;
-export type Builder_CompileJob_t = { cFile: string, outputFile: ?string };
+export type Builder_CompileJob_t = {
+    cFile: string,
+    outputFile: ?string,
+    type: 'exe'|'lib'
+};
+export type Builder_Pub_t = {|
+    build: (Builder_Stage_t)=>Builder_Pub_t,
+    test: (Builder_Stage_t)=>Builder_Pub_t,
+    pack: (Builder_Stage_t)=>Builder_Pub_t,
+    failure: (Builder_Stage_t)=>Builder_Pub_t,
+    success: (Builder_Stage_t)=>Builder_Pub_t,
+    complete: (Builder_Stage_t)=>Builder_Pub_t,
+|}
 export type Builder_PreCtx_t = {
     buildStage: Builder_Stage_t,
     testStage: Builder_Stage_t,
@@ -182,7 +195,7 @@ const cc = function (
 ) {
     compiler(ctx, args, function (ret, out, err) {
         if (ret) {
-            return callback(error("gcc " + args.map(String).join(' ') + "\n\n" + err));
+            return callback(error(ctx.config.gcc + " " + args.map(String).join(' ') + "\n\n" + err));
         }
 
         if (err !== '') {
@@ -195,9 +208,6 @@ const cc = function (
 
 const getStatePrototype = function () /*:Builder_State_t*/ {
     return {
-        rebuildIfChanges: [],
-        rebuildIfChangesHash: '',
-
         compilerType: {
             isLLVM: false,
             isClang: false,
@@ -226,13 +236,17 @@ const finalizeCtx = function (
             compiler(ctx, args, callback, '');
         },
 
+        buildLibrary: function (cFile) {
+            ctx.executables.push({ cFile, outputFile: null, type: 'lib' });
+        },
+
         buildExecutable: function (cFile, outputFile) {
-            ctx.executables.push({ cFile, outputFile });
+            ctx.executables.push({ cFile, outputFile, type: 'exe' });
         },
 
         buildTest: function (cFile) {
             const outputFile = getTempExe(ctx, cFile);
-            ctx.executables.push({ cFile, outputFile });
+            ctx.executables.push({ cFile, outputFile, type: 'exe' });
             return outputFile;
         },
 
@@ -252,6 +266,8 @@ const finalizeCtx = function (
 
         compilerType: () => JSON.parse(JSON.stringify(ctx.state.compilerType)),
 
+        fileCflags: {},
+
     }) /*:Builder_t*/);
     return ctx;
 };
@@ -424,10 +440,26 @@ const getExeFile = function (ctx, exe /*:Builder_CompileJob_t*/) {
     return outputFile;
 };
 
+var randomHex = function (bytes, fileName) {
+    var material = new Crypto.Hash('sha512').update(seed).update(fileName).digest();
+    if (bytes > 64) { throw new Error("meh, randomHex of over 64 bytes is unimplemented"); }
+    return material.slice(0, bytes).toString('hex');
+};
+
 const getFlags = function (ctx, cFile, includeDirs) {
     const flags = [];
-    flags.push.apply(flags, ctx.config.cflags);
-    flags.push.apply(flags, ctx.builder.config.fileCflags[cFile] || []);
+    if (cFile.indexOf('node_build/dependencies/libuv') > -1) {
+        //console.log('cargo:warning=' + cFile);
+        for (const f of ctx.config.cflags) {
+            if (f !== '-Werror') {
+                flags.push(f);
+            }
+        }
+    } else {
+        flags.push.apply(flags, ctx.config.cflags);
+    }
+    flags.push.apply(flags, ctx.builder.fileCflags[cFile] || []);
+    flags.push('-DCJDNS_RAND_U64_PER_FILE=0x' + randomHex(8, cFile) + 'ull');
     if (includeDirs) {
         for (let i = 0; i < ctx.config.includeDirs.length; i++) {
             if (flags[flags.indexOf(ctx.config.includeDirs[i])-1] === '-I') {
@@ -564,19 +596,29 @@ const getLinkOrder = function (cFile, files) /*:string[]*/ {
 // make sure everything is present.
 const needsToLink = function (ctx, cFile) {
     const nlCache = {};
+    const nll = [];
     const nl = (cFile) => {
         if (nlCache[cFile]) { return false; }
-        //debug('  ' + cFile);
-        if (typeof(ctx.state.cFiles[cFile]) !== 'object') {
-            return true;
+        if (nll.indexOf(cFile) > -1) {
+            return false;
+            //throw new Error(`File ${cFile} is self-referencial:\n${nll.join('\n')}\n\n`);
         }
-        for (const l of ctx.state.cFiles[cFile].links) {
-            if (l !== cFile && nl(l)) {
+        nll.push(cFile);
+        const out = (() => {
+            //debug('  ' + cFile);
+            if (typeof(ctx.state.cFiles[cFile]) !== 'object') {
                 return true;
             }
-        }
-        nlCache[cFile] = true;
-        return false;
+            for (const l of ctx.state.cFiles[cFile].links) {
+                if (l !== cFile && nl(l)) {
+                    return true;
+                }
+            }
+            nlCache[cFile] = true;
+            return false;
+        })();
+        if (nll.pop() !== cFile) { throw new Error(); }
+        return out;
     };
     return nl(cFile);
 };
@@ -753,7 +795,7 @@ const sweep = (path, done) => {
 module.exports.configure = function (
     params /*:Builder_BaseConfig_t*/,
     configFunc /*:(Builder_t, Nthen_WaitFor_t)=>void*/
-) {
+) /*:Builder_Pub_t*/ {
 
     // Track time taken for various steps
     const time = makeTime();
@@ -806,7 +848,6 @@ module.exports.configure = function (
             cflags: [],
             ldflags: [],
             libs: [],
-            fileCflags: {},
             jobs,
         },
     };
@@ -839,6 +880,11 @@ module.exports.configure = function (
 
     }).nThen(function (waitFor) {
 
+        if (process.env['CJDNS_FULL_REBUILD']) {
+            debug("CJDNS_FULL_REBUILD set, non-incremental build");
+            return;
+        }
+
         // read out the state if it exists
         Fs.exists(buildDir + '/state.json', waitFor(function (exists) {
             if (!exists) { return; }
@@ -850,15 +896,6 @@ module.exports.configure = function (
             }));
         }));
 
-    }).nThen(function (waitFor) {
-
-        getRebuildIfChangesHash(state.rebuildIfChanges, waitFor(function (rich) {
-            if (rich !== state.rebuildIfChangesHash) {
-                debug("rebuildIfChanges changed, rebuilding everything");
-                state.cFiles = {};
-            }
-        }));
-
     }).nThen(function (waitFor) {
 
         debug("Initialize " + time() + "ms");
@@ -1006,17 +1043,6 @@ const postConfigure = (ctx /*:Builder_Ctx_t*/, time) => {
     const state = ctx.state;
     nThen((waitFor) => {
 
-        if (state.rebuildIfChanges.indexOf(module.parent.filename) === -1) {
-            // Always always rebuild if the makefile was changed.
-            state.rebuildIfChanges.push(module.parent.filename);
-        }
-
-        getRebuildIfChangesHash(state.rebuildIfChanges, waitFor(function (rich) {
-            state.rebuildIfChangesHash = rich;
-        }));
-
-    }).nThen(function (waitFor) {
-
         removeStaleFiles(ctx, waitFor());
 
     }).nThen(function (waitFor) {
@@ -1055,7 +1081,9 @@ const postConfigure = (ctx /*:Builder_Ctx_t*/, time) => {
     }).nThen(function (waitFor) {
         debug("Compile " + time() + "ms");
         for (const exe of ctx.executables) {
-            link(exe.cFile, waitFor(), ctx);
+            if (exe.type === 'exe') {
+                link(exe.cFile, waitFor(), ctx);
+            }
         }
     }).nThen((w) => {
         debug("Link " + time() + "ms");
