@@ -23,6 +23,7 @@
 #include "util/events/Time.h"
 
 #include <stdbool.h>
+#include <inttypes.h>
 
 #define TIMEOUT_MILLISECONDS 10000
 
@@ -70,7 +71,7 @@ struct ReachabilityCollector_pvt
     struct Log* log;
     struct BoilerplateResponder* br;
     struct Address* myAddr;
-    struct EventBase* base;
+    EventBase_t* base;
     struct EncodingScheme* myScheme;
     struct SwitchPinger* sp;
     uint32_t resampleCycle;
@@ -106,7 +107,7 @@ static struct PeerInfo_pvt* piForAddr(struct ReachabilityCollector_pvt* rcp, str
     return NULL;
 }
 
-static int piOnFree(struct Allocator_OnFreeJob* j)
+static void piOnFree(struct Allocator_OnFreeJob* j)
 {
     struct PeerInfo_pvt* pi = Identity_check((struct PeerInfo_pvt*) j->userData);
     struct ReachabilityCollector_pvt* rcp = Identity_check(pi->rcp);
@@ -115,7 +116,6 @@ static int piOnFree(struct Allocator_OnFreeJob* j)
         if (pi0 != pi) { continue; }
         ArrayList_OfPeerInfo_pvt_remove(rcp->piList, j);
     }
-    return 0;
 }
 
 static void mkNextRequest(struct ReachabilityCollector_pvt* rcp);
@@ -202,7 +202,7 @@ static void latencyUpdate(
     //Log_debug(rcp->log, "Latency update for [%" PRIx64 "] [%u]ms", pip->pub.addr.path, lag);
     pip->sumOfLag += lag;
     pip->lagSamples++;
-    pip->timeOfLastLagUpdate = Time_currentTimeMilliseconds(rcp->base);
+    pip->timeOfLastLagUpdate = Time_currentTimeMilliseconds();
 }
 
 static void onReplyOld(Dict* msg, struct Address* src, struct MsgCore_Promise* prom)
@@ -494,6 +494,12 @@ struct ReachabilityCollector_PeerInfo*
     return pi ? &pi->pub : NULL;
 }
 
+int ReachabilityCollector_peerCount(struct ReachabilityCollector* rc)
+{
+    struct ReachabilityCollector_pvt* rcp = Identity_check((struct ReachabilityCollector_pvt*) rc);
+    return rcp->piList->length;
+}
+
 void ReachabilityCollector_lagSample(
     struct ReachabilityCollector* rc, uint64_t label, uint32_t milliseconds)
 {
@@ -530,7 +536,7 @@ static void dummyOnChange(
 struct ReachabilityCollector* ReachabilityCollector_new(struct Allocator* allocator,
                                                         struct MsgCore* msgCore,
                                                         struct Log* log,
-                                                        struct EventBase* base,
+                                                        EventBase_t* base,
                                                         struct BoilerplateResponder* br,
                                                         struct Address* myAddr,
                                                         struct EncodingScheme* myScheme,
