@@ -12,18 +12,18 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
-#include "util/Setuid_impl.h"
+#include "util/Setuid.h"
 #include "memory/Allocator.h"
-#include "exception/Er.h"
+#include "exception/Err.h"
 
 // This implementation does nothing
 
-Er_DEFUN(void Setuid_preSetuid(struct Allocator* alloc))
+Err_DEFUN Setuid_preSetuid(struct Allocator* alloc)
 {
-    Er_ret();
+    return NULL;
 }
 
-Er_DEFUN(void Setuid_postSetuid(struct Allocator* alloc))
+Err_DEFUN Setuid_postSetuid(struct Allocator* alloc)
 {
-    Er_ret();
+    return NULL;
 }
