@@ -15,21 +15,17 @@
 #ifndef EventBase_H
 #define EventBase_H
 
+#include "rust/cjdns_sys/RTypes.h"
 #include "memory/Allocator.h"
 #include "util/Linker.h"
 Linker_require("util/events/libuv/EventBase.c")
 
-struct EventBase
-{
-    int unused;
-};
+typedef RTypes_EventLoop_t EventBase_t;
 
-struct EventBase* EventBase_new(struct Allocator* alloc);
+EventBase_t* EventBase_new(struct Allocator* alloc);
 
-int EventBase_eventCount(struct EventBase* eventBase);
+void EventBase_beginLoop(EventBase_t* eventBase);
 
-void EventBase_beginLoop(struct EventBase* eventBase);
-
-void EventBase_endLoop(struct EventBase* eventBase);
+void EventBase_endLoop(EventBase_t* eventBase);
 
 #endif
